!===================================================================================
SUBROUTINE dyn_mat( dyn, latt, sh, q, ntot, list )
  !=================================================================================
  USE nrtype
  USE nrutil
  USE diagonalize
  USE data
  IMPLICIT NONE

  LOGICAL :: first=.true.
  SAVE :: first
  LOGICAL :: finns

  INTEGER :: na, nb, nc, nr, nbranches, i, ntot, list(3,ntot), naa, nbb, natoms, ntyper,i1, i2, i3,MESHSIZE

  DOUBLE PRECISION, ALLOCATABLE :: BORNECHARG(:,:,:)
  DOUBLE PRECISION :: BORN1(3,3),BORN2(3,3),DIELECTR(3,3),PRIMCELLVOL,GG(3),KK(3),COMP1(3),COMP2(3)
  DOUBLE PRECISION :: MATRIXA1(3,3),MATRIXA2(3,3),BORN3(3,3),COMP3(3),COMP4(3)
  TYPE(dynmat), INTENT(INOUT) :: dyn
  TYPE(dynmat) :: dcorr
  TYPE(lattice), INTENT(IN) :: latt
  TYPE(shell), INTENT(IN)  :: sh

  REAL(DP), INTENT(IN) :: q(3)
  REAL(DP) :: temp(3), x(3), fac,xx(3)
  REAL(DP), PARAMETER :: eps = 1.d-6
  first=.true.
   !=====================================================================
  ! On input, the forces are in eV/A and the distances in A.
  ! Conversion factor to THZ**2
  !=====================================================================

  fac = evtoj   &           !  D^2 U eV  --> Joule
       /1.d-20  &        !  D^2 R A^2 --> meter 
       /amtokg  &        !  proton mass --> Kg
       /1.d24/twopi**2   !  cicles in THZ^2

  nbranches = 3*latt%natoms

  if(first) then
     allocate( dyn%cmat(nbranches,nbranches), dyn%eig(nbranches))
     first = .false.
  endif

  dyn%cmat = 0
  dyn%eig = 0
 
  do i = 1, ntot
     na = list(1,i)
     nb = list(2,i)
     nr = list(3,i)

!     if( dyn%weight(na,nb,nr) > 0 ) then

     x = sh%r(:,nr) + latt%x(:,nb) - latt%x(:,na)

        dyn%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) = &
             dyn%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) &
             + ( dyn%dmat(:,:,na,nb,nr) * exp(  -im * dot_product( q, x ) ) ) / &
             sqrt( latt%mass( latt%ityp(na) ) * latt%mass(latt%ityp(nb)) )

!     endif

  enddo

  dyn%cmat = dyn%cmat * fac

  !----------------------------------
  ! Here the Born-effective charge tensors and the dielectric matrices are 
  ! loaded from the file DIPOLEINTERACT if it exist. If the file exsist the
  ! long-range dipole correctional term is calculated as it is described in
  ! Equation (75) page 77 of PRB 55, 10355 (1997)
  !--------------------------------------

  inquire(file='DIPOLEINTERACT',exist=finns) 
  IF (finns) THEN
     ntyper = latt%natoms
     allocate(dcorr%cmat(nbranches,nbranches),BORNECHARG(ntyper,3,3))
     dcorr%cmat(:,:)= (0.0d0,0.0d0)
     OPEN(3489,FILE='DIPOLEINTERACT',STATUS='OLD',ACTION='READ')
     READ(3489,*)MESHSIZE
     READ(3489,*)PRIMCELLVOL
     READ(3489,*)DIELECTR(1,1),DIELECTR(1,2),DIELECTR(1,3)
     READ(3489,*)DIELECTR(2,1),DIELECTR(2,2),DIELECTR(2,3)
     READ(3489,*)DIELECTR(3,1),DIELECTR(3,2),DIELECTR(3,3)
     DO i=1,ntyper
        READ(3489,*)BORNECHARG(i,1,1),BORNECHARG(i,1,2),BORNECHARG(i,1,3)
        READ(3489,*)BORNECHARG(i,2,1),BORNECHARG(i,2,2),BORNECHARG(i,2,3)
        READ(3489,*)BORNECHARG(i,3,1),BORNECHARG(i,3,2),BORNECHARG(i,3,3)
     ENDDO
     close(3489)
     DO na=1,ntyper
      DO nb=1,ntyper
       BORN1(:,:) = BORNECHARG(na,:,:)
       BORN2(:,:) = BORNECHARG(nb,:,:)
       DO i1=-MESHSIZE,MESHSIZE
        DO i2=-MESHSIZE,MESHSIZE
         DO i3=-MESHSIZE,MESHSIZE
          x = latt%x(:,nb) - latt%x(:,na)
          GG(1) = i1*1.0d0/(2.0d0*MESHSIZE)
          GG(2) = i2*1.0d0/(2.0d0*MESHSIZE)
          GG(3) = i3*1.0d0/(2.0d0*MESHSIZE)
          GG = twopi/latt%scale*matmul(latt%bg,GG)
          KK = GG + q
          COMP1 = MATMUL(KK,BORN1)
          COMP2 = MATMUL(KK,BORN2)
          MATRIXA1(1,:) = COMP1(1)*COMP2
          MATRIXA1(2,:) = COMP1(2)*COMP2
          MATRIXA1(3,:) = COMP1(3)*COMP2
         IF ( dot_product(KK,MATMUL(DIELECTR,KK)) .GT. 0.0000001 ) THEN
            IF ( i1 .NE. 0 .AND. i2 .NE. 0 .AND. i3 .NE. 0 ) THEN
             dcorr%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) = &
             dcorr%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) &
             + (MATRIXA1(:,:)/( dot_product(KK,MATMUL(DIELECTR,KK)) ))*exp(  im * dot_product( KK, x ) ) &
             /sqrt( latt%mass( latt%ityp(na) ) * latt%mass(latt%ityp(nb)) )
            ENDIF
         ENDIF
         IF ( na .EQ. nb .AND. i1 .NE. 0 .AND. i2 .NE. 0 .AND. i3 .NE. 0 ) THEN
            DO nc=1,ntyper
               BORN3(:,:) = BORNECHARG(nc,:,:)
               xx = latt%x(:,nb) - latt%x(:,nc)

               COMP3 = MATMUL(GG,BORN1)
               COMP4 = MATMUL(GG,BORN3)

               MATRIXA2(1,:) = COMP3(1)*COMP4
               MATRIXA2(2,:) = COMP3(2)*COMP4
               MATRIXA2(3,:) = COMP3(3)*COMP4
               IF (dot_product(GG,MATMUL(DIELECTR,GG)) .GT. 0.0000001 ) THEN
                dcorr%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) = &
                dcorr%cmat( 1 + (na-1)*3 : 3 + (na-1)*3, 1 + (nb-1)*3 : 3 + (nb-1)*3 ) &
                -(MATRIXA2(:,:)/( dot_product(GG,MATMUL(DIELECTR,GG)) ))*exp(  im * dot_product( GG, xx ) ) &
                /sqrt( latt%mass( latt%ityp(na) ) * latt%mass(latt%ityp(nb)) )
              ENDIF
            ENDDO
         ENDIF
        ENDDO
       ENDDO
      ENDDO
     ENDDO
    ENDDO
    !dcorr%cmat = 0.5*(TRANSPOSE(CONJG(dcorr%cmat))+dcorr%cmat)
    dcorr%cmat = dcorr%cmat*2.0d0*twopi/(PRIMCELLVOL)!*(2*MESHSIZE+1)**3)
    dcorr%cmat = dcorr%cmat*(1602.0/8.8541878)  ! ---> eV/(atomic mass unit * angstroem**2)
    dcorr%cmat = dcorr%cmat * fac   ! eV/(atomic mass unit * angstroem**2) ---> THz**2
    !print*,dcorr%cmat
    !STOP
  ENDIF
 
  IF (finns) dyn%cmat = dyn%cmat + dcorr%cmat
  IF (finns) deallocate(dcorr%cmat,BORNECHARG)
  !----------------------------
  ! Check Hermiticity
  !----------------------------
  temp = 0
  do na=1, nbranches
     do nb=1, nbranches
        if( nb /= na ) then
           temp(1) = temp(1) + abs( dyn%cmat(na,nb) - conjg(dyn%cmat(nb,na) ) )
        endif
        if( nb == na ) temp(2) = temp(2) + abs( dyn%cmat(na,na) - conjg(dyn%cmat(na,na) ) )
        temp(3) = temp(3) +  abs( dyn%cmat(na,nb) )
     enddo
  enddo
  if( abs(temp(1)) > eps .or. abs(temp(2)) > eps ) &
       print'(''Dynamical matrix not hermitean '',3e9.1,2x,2e9.1)', &
       temp(1),temp(2),temp(3),temp(1)/temp(3),temp(2)/temp(3)


END SUBROUTINE dyn_mat
